package com.tzld.piaoquan.recommend.server.task.consumer;

import com.google.gson.JsonObject;
import com.tzld.piaoquan.recommend.server.task.remote.MilvusRemoteService;
import com.tzld.piaoquan.recommend.server.task.remote.ModelRemoteService;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.apis.ClientConfiguration;
import org.apache.rocketmq.client.apis.ClientConfigurationBuilder;
import org.apache.rocketmq.client.apis.ClientException;
import org.apache.rocketmq.client.apis.ClientServiceProvider;
import org.apache.rocketmq.client.apis.consumer.*;
import org.apache.rocketmq.client.apis.message.MessageView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.nio.ByteBuffer;
import java.util.*;

/**
 * @author dyp
 */
@Component
@Slf4j
public class VideoVectorProduceConsumer {


    @Autowired
    private ModelRemoteService modelRemoteService;

    @Autowired
    private MilvusRemoteService milvusRemoteService;

    @PostConstruct
    public void init() {
        try {
            start();
        } catch (Exception e) {
            log.error("", e);
        }
    }

    private void start() throws ClientException {
        /**
         * 实例接入点，从控制台实例详情页的接入点页签中获取。
         * 如果是在阿里云ECS内网访问，建议填写VPC接入点。
         * 如果是在本地公网访问，或者是线下IDC环境访问，可以使用公网接入点。使用公网接入点访问，必须开启实例的公网访问功能。
         */
        String endpoints = "rmq-cn-xxx.{regionId}.rmq.aliyuncs.com:8080";
        //指定需要订阅哪个目标Topic，Topic需要提前在控制台创建，如果不创建直接使用会返回报错。
        String topic = "Your Topic";
        //为消费者指定所属的消费者分组，Group需要提前在控制台创建，如果不创建直接使用会返回报错。
        String consumerGroup = "Your ConsumerGroup";
        ClientServiceProvider provider = ClientServiceProvider.loadService();
        ClientConfigurationBuilder builder = ClientConfiguration.newBuilder().setEndpoints(endpoints);
        /**
         * 如果是使用公网接入点访问，configuration还需要设置实例的用户名和密码。用户名和密码在控制台访问控制的智能身份识别页签中获取。
         * 如果是在阿里云ECS内网访问，无需填写该配置，服务端会根据内网VPC信息智能获取。
         * 如果实例类型为Serverless实例，公网访问必须设置实例的用户名密码，当开启内网免身份识别时,内网访问可以不设置用户名和密码。
         */
        //builder.setCredentialProvider(new StaticSessionCredentialsProvider("Instance UserName", "Instance Password"));
        ClientConfiguration clientConfiguration = builder.build();
        //订阅消息的过滤规则，表示订阅所有Tag的消息。
        String tag = "*";
        FilterExpression filterExpression = new FilterExpression(tag, FilterExpressionType.TAG);
        //初始化PushConsumer，需要绑定消费者分组ConsumerGroup、通信参数以及订阅关系。
        PushConsumer pushConsumer = provider.newPushConsumerBuilder()
                .setClientConfiguration(clientConfiguration)
                //设置消费者分组。
                .setConsumerGroup(consumerGroup)
                //设置预绑定的订阅关系。
                .setSubscriptionExpressions(Collections.singletonMap(topic, filterExpression))
                //设置消费监听器。
                .setMessageListener(new Listener())
                .build();
    }

    public class Listener implements MessageListener {

        @Override
        public ConsumeResult consume(MessageView messageView) {
            try {
                ByteBuffer messageBody = messageView.getBody();
                String messageContent = new String(messageView.getBody().array());
                // TODO 1. obtain video
                
                // TODO 2. obtain feature

                // TODO 3. trans vector
                transVector();
                // TODO 4. write milvus
                saveVector();
            } catch (Exception e) {

            }
            return ConsumeResult.SUCCESS;
        }

        private float[] transVector() {
            return modelRemoteService.dssmPredict("");
        }

        private void saveVector() {
            List<JsonObject> datas = new ArrayList<>();
            JsonObject obj = new JsonObject();
            obj.addProperty("id", 0L);
            obj.addProperty("vector", "[0.3580376395471989f, -0.6023495712049978f, 0.18414012509913835f, -0" +
                    ".26286205330961354f, 0.9029438446296592f]");
            obj.addProperty("color", "pink");
            datas.add(obj);
            milvusRemoteService.insert("demo", datas);
        }
    }


}
